'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQLGateway = undefined;

var _graphqlTools = require('graphql-tools');

var _graphql = require('graphql');

var _lodash = require('lodash.difference');

var _lodash2 = _interopRequireDefault(_lodash);

var _selectn = require('selectn');

var _selectn2 = _interopRequireDefault(_selectn);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _createRemoteSchema = require('./createRemoteSchema');

var _buildRelationalResolvers = require('./buildRelationalResolvers');

var _utilities = require('./utilities');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class GraphQLGateway {

  constructor(opts) {
    this.blacklist = ['$node'];
    this.discoveredTypes = {};
    this.expectedTypes = [];
    this.generateSnapshot = false;
    this.initialized = false;
    this.relationDefinitions = {};
    this.relationships = {};
    this.remoteSchemas = {};
    this.schema = null;
    this.service = null;
    this.snapshotPath = `${process.cwd()}/schema.snapshot.graphql`;
    this.waitTimeout = 5000;
    this.waitInterval = 100;

    this.handleServiceUpdate = async opts => {
      const services = this.broker.services.filter(service => service.settings.hasGraphQLSchema).filter(service => !this.blacklist.includes(service.name)).filter(service => !this.discoveredTypes[service.settings.typeName]);

      if (services.length > 0) {
        for (const service of services) {
          this.discoveredTypes[service.settings.typeName] = service.name;
          await this.buildRemoteSchema(service);
          if (this.onServiceDiscovery) {
            this.onServiceDiscovery(service);
          }
        }
        this.generateSchema();
      }
    };

    this.handleNodeConnection = async ({ node }) => {
      const services = node.services.filter(service => !this.discoveredTypes[service.settings.typeName] && !this.blacklist.includes(service.name) && service.settings.hasGraphQLSchema);
      if (services.length > 0) {
        for (const service of services) {
          this.discoveredTypes[service.settings.typeName] = service.name;
          await this.buildRemoteSchema(service);
          if (this.onServiceDiscovery) {
            this.onServiceDiscovery(service);
          }
        }
        this.generateSchema();
      }
    };

    this.handleNodeDisconnected = async ({ node }) => {
      const services = node.services.filter(service => this.remoteSchemas[service.settings.typeName]);
      if (services.length > 0) {
        for (const service of services) {
          await this.buildRemoteSchema(service);
        }
        this.generateSchema();
      }
    };

    this.broker = opts.broker;
    if (opts.expectedTypes) this.expectedTypes = opts.expectedTypes;
    if (opts.waitInterval) this.waitInterval = opts.waitInterval;
    if (opts.waitTimeout) this.waitTimeout = opts.waitTimeout;
    if (opts.blacklist) this.blacklist.concat(opts.blacklist);
    if (opts.generateSnapshot) this.generateSnapshot = opts.generateSnapshot;
    if (opts.snapshotPath) this.snapshotPath = opts.snapshotPath;
    if (opts.onServiceDiscovery) this.onServiceDiscovery = opts.onServiceDiscovery;
    this.service = this.broker.createService({
      name: 'gateway',
      events: {
        '$services.changed': this.handleServiceUpdate,
        '$node.connected': this.handleNodeConnection,
        '$node.disconnected': this.handleNodeDisconnected
      }
    });
  }

  alphabetizeSchema(schema) {
    const queryType = schema._queryType;
    const fields = queryType.getFields();
    const unordered = Object.keys(fields);
    const ordered = Object.keys(fields).sort();
    if (JSON.stringify(unordered) !== JSON.stringify(ordered)) {
      const alphabetized = {};
      ordered.forEach(field => {
        alphabetized[field] = fields[field];
      });
      queryType._fields = alphabetized;
      schema._queryType = queryType;
    }
    return schema;
  }

  async buildRemoteSchema(service) {
    const { settings: { typeName, relationships, relationDefinitions } } = service;
    if (!this.remoteSchemas[typeName]) {
      this.remoteSchemas[typeName] = await (0, _createRemoteSchema.createRemoteSchema)({
        broker: this.broker,
        service
      });
      if (relationships) {
        this.relationships[typeName] = relationships;
        this.relationDefinitions[typeName] = relationDefinitions;
        const relatedTypes = (0, _utilities.getRelatedTypes)((0, _graphql.parse)(relationships));
        const missingTypes = (0, _lodash2.default)(relatedTypes, this.discoveredTypes);
        this.expectedTypes = this.expectedTypes.concat(missingTypes);
      }
    }
  }

  generateSchema() {
    const schemas = Object.values(this.remoteSchemas).concat(Object.values(this.relationships));
    const resolvers = (0, _buildRelationalResolvers.buildRelationalResolvers)(this.relationDefinitions);
    this.schema = (0, _graphqlTools.mergeSchemas)({
      schemas,
      resolvers
    });
    this.schema = this.alphabetizeSchema(this.schema);
    if (this.generateSnapshot) this.recordSnapshot();
    return this.schema;
  }

  recordSnapshot() {
    if (this.schema) {
      _fs2.default.writeFileSync(this.snapshotPath, (0, _graphql.printSchema)(this.schema));
    }
  }

  start() {
    return new Promise((resolve, reject) => {
      const maxTries = this.waitTimeout / this.waitInterval;
      let tries = 0;
      this.timer = setInterval(() => {
        tries++;
        if (tries >= maxTries) {
          reject(new Error('Timeout'));
        }
        const discoveredTypes = Object.keys(this.discoveredTypes);
        const undiscovered = (0, _lodash2.default)(this.expectedTypes, discoveredTypes);
        if (discoveredTypes.length === 0) return;
        if (discoveredTypes.some(type => !this.remoteSchemas[type])) return;
        if (undiscovered.length > 0) {
          if (this.broker.logger) {
            const msg = `Still waiting for ${undiscovered.join(', ')} types to be discovered`;
            this.broker.logger.warn(msg);
          }
          return;
        }
        clearInterval(this.timer);
        this.generateSchema();
        resolve(this.schema);
      }, this.waitInterval);
    });
  }
}
exports.GraphQLGateway = GraphQLGateway;